"use strict";

var MinervaPluginProxy = require('./MinervaPluginProxy');
var ObjectWithListeners = require('../ObjectWithListeners');

var Promise = require("bluebird");

var logger = require('../logger');
var Functions = require('../Functions');

var pluginId = 0;

function Plugin(options) {
  ObjectWithListeners.call(this);
  var self = this;
  self.setOptions(options);
  self.registerListenerType("onUnload");
}

Plugin.prototype = Object.create(ObjectWithListeners.prototype);
Plugin.prototype.constructor = ObjectWithListeners;

Plugin.prototype.setOptions = function (options) {
  this._options = options;
};

Plugin.prototype.getOptions = function () {
  return this._options;
};

Plugin.prototype.setLoadedPluginData = function (loadedPluginData) {
  this._loadedPluginData = loadedPluginData;
};
Plugin.prototype.getLoadedPluginData = function () {
  return this._loadedPluginData;
};

Plugin.prototype.setMinervaPluginProxy = function (minervaPluginProxy) {
  this._minervaPluginProxy = minervaPluginProxy;
};
Plugin.prototype.getMinervaPluginProxy = function () {
  return this._minervaPluginProxy;
};
Plugin.prototype.getPluginId = function () {
  return this.getMinervaPluginProxy().pluginId;
};

Plugin.prototype.load = function () {
  var self = this;
  var options = self.getOptions();


  return ServerConnector.sendRequest({
    url: options.url,
    description: "Loading plugin: " + options.url,
    method: "GET"
  }).then(function (content) {
    var pluginData = undefined;
    var error = false;
    try {
      // noinspection JSUnusedLocalSymbols
      var define = function (pluginFunction) {
        try {
          if (typeof pluginFunction === "function") {
            pluginData = pluginFunction();
          } else {
            pluginData = pluginFunction;
          }

          var minervaPluginProxy = new MinervaPluginProxy({
            map: options.map,
            configuration: options.configuration,
            element: options.element,
            pluginId: "plugin" + (pluginId++)
          });
          self.setLoadedPluginData(pluginData);
          self.setMinervaPluginProxy(minervaPluginProxy);
          pluginData.register(minervaPluginProxy);
        } catch (e) {
          error = e;
        }
      };
      eval(content);
    } catch (e) {
      error = e;
    }
    if (error) {
      return Promise.reject(error);
    }
  });

};

Plugin.prototype.unload = function () {
  var self = this;
  return Promise.resolve().then(function () {
    return self.getLoadedPluginData().unregister();
  }).then(function () {
    var removedListeners = self.getMinervaPluginProxy().project.map.removeAllListeners();
    if (removedListeners.length > 0) {
      logger.warn("'"+self.getLoadedPluginData().getName() + "' plugin didn't remove all registered listeners");
    }
    return self.callListeners("onUnload");
  }).then(function () {
    return self.destroy();
  });
};

Plugin.prototype.getName = function () {
  return this.getLoadedPluginData().getName();
};

Plugin.prototype.getVersion = function () {
  return this.getLoadedPluginData().getVersion();
};

Plugin.prototype.destroy = function () {
  return Promise.resolve();
};

module.exports = Plugin;
